#!/usr/bin/python
"""Read and write on a advisory lock tests. """

from RECT import *


class SingleReadAndWriteOnAdvLock(TestCase):
    """Read and write on a lock tests with one slave (advisory locks)
    (thread).

    Provided tests list:

    1. Write on a read lock (Allowed)
    2. Write on a write lock (Allowed)
    3. Read on a read lock (Allowed)
    4. Read on a write lock (Allowed)

    Tests written accoodinately of a advisory locking type
    """
    n_shares = 1
    n_slaves = 1

    def setUp(self):
        TestCase.setUp(self)
        self.filename = "%s/%s" % (self.shares[0],
                                   self.config.get('filename', 'test.dat'))
        self.data = self.config.get('data', 'Example data')

    def testWriteOnReadLock(self):
        """Write on a read lock (Allowed)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave0: Open file for writing, write data and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file.close()

        file1 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        file2 = slave.open(self.filename, OpenMode.WRITE, CreationMode.OPEN)
        file2.write(self.data)

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


    def testWriteOnWriteLock(self):
        """Write on a write lock (Allowed)

        Slave0: Create a file, set a write lock
        Slave0: Open file for writing, write data and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file1 = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        file2 = slave.open(self.filename, OpenMode.WRITE, CreationMode.OPEN)
        file2.write(self.data)

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


    def testReadOnReadLock(self):
        """Read on a read lock (Allowed)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave0: Open file for reading, read data and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file.close()

        file1 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        file2 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        res = file2.read(len(self.data) + 500)

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


    def testReadOnWriteLock(self):
        """Read on a write lock (Allowed)

        Slave0: Create a file, set a write lock
        Slave0: Open file for reading, read data and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file1 = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        file2 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        res = file2.read(len(self.data) + 500)

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


class DualReadAndWriteOnAdvLock(TestCase):
    """Read and write on a lock tests with two slaves (advisory locks).
    (process)

    Provided tests list:

    1. Write on a read lock (Allowed)
    2. Write on a write lock (Allowed)
    3. Read on a read lock (Allowed)
    4. Read on a write lock (Allowed)

    Tests written accoodinately of a advisory locking type
    """
    n_shares = 1
    n_slaves = 2

    def setUp(self):
        TestCase.setUp(self)
        self.filename = "%s/%s" % (self.shares[0],
                                   self.config.get('filename', 'test.dat'))
        self.data = self.config.get('data', 'Example data')

    def testWriteOnReadLock(self):
        """Write on a read lock (Allowed)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave1: Open file for writing, write data and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                   CreationMode.CREATE)
        file.close()

        file1 = self.slaves[0].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.WRITE,
                                    CreationMode.OPEN)
        file2.write(self.data)

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


    def testWriteOnWriteLock(self):
        """Write on a write lock (Allowed)

        Slave0: Create a file, set a write lock
        Slave1: Open file for writing, write data and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file1 = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                    CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.WRITE,
                                    CreationMode.OPEN)
        file2.write(self.data)

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


    def testReadOnReadLock(self):
        """Read on a read lock (Allowed)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave1: Open file for reading, read data and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                   CreationMode.CREATE)
        file.close()

        file1 = self.slaves[0].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        res = file2.read(len(self.data) + 500)

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


    def testReadOnWriteLock(self):
        """Read on a write lock (Allowed)

        Slave0: Create a file, set a write lock
        Slave1: Open file for reading, read data and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file1 = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                    CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        res = file2.read(len(self.data) + 500)

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


if __name__ == '__main__':
    main()

